/*
** Module   : DialectHelper.java
** Abstract : a helper to P2J dialect classes
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 VMN 20131012 Created initial version.
*/

package com.goldencode.p2j.persist.dialect;

import java.net.*;
import com.goldencode.p2j.persist.*;

/**
 * A helper to P2J dialect classes. This class provides static helper methods and constants used for
 * P2J dialects.
 */
public class DialectHelper
{
   /** Prefix prepended to computed column/property names */
   public static final String CC_PREFIX = "__";

   /** Double quote character */
   public static final char DOUBLE_QUOTE = '"';

   /**
    * Build the URL to be used for a remote connection, based on the given
    * URL and database.
    *
    * @param    url
    *           The URL as received from the remote server.
    * @param    database
    *           The database to which this URL belongs.
    *
    * @return   The built URL as to be used for a remote connection.
    *
    * @throws com.goldencode.p2j.persist.PersistenceException
    *           if the specified connection URL is invalid.
    */
   public static String buildRemoteURL(String url, Database database)
   throws PersistenceException
   {
      String origUrl = url;
      try
      {
         url = url.substring("jdbc:".length());
         URI uri = (new URI(url)).normalize();

         // Check for localhost references.
         String host = uri.getHost().toLowerCase();
         if (InetAddress.getByName(host).isLoopbackAddress())
         {
            // Use hostname stored in database object.
            host = database.getSocketAddress().getHostName();

            // Generate new URI.
            uri = new URI(uri.getScheme(),
                    uri.getUserInfo(),
                    host,
                    uri.getPort(),
                    uri.getPath(),
                    uri.getQuery(),
                    uri.getFragment());

            // Prepend 'jdbc:' prefix.
            StringBuilder buf = new StringBuilder("jdbc:");
            buf.append(uri);
            url = buf.toString();
         }
         else
         {
            // no preparation is needed
            url = origUrl;
         }
      }
      catch (Exception exc)
      {
         throw new PersistenceException("Invalid database connection URL", exc);
      }

      return url;
   }
}
