/*
** Module   : SQLServer2012SequenceHandler.java
** Abstract : Static class that handles sequence functions and statements 
**            from Progress / OpenEdge 10.x.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 VMN 20131012 Created initial stub version.
*/

package com.goldencode.p2j.persist.sequence;

import java.util.*;
import com.goldencode.p2j.util.*;

/**
 * SQL Server 2012 implementation of sequences using a native SQL sequence primitives queries.
 */
public class SQLServer2012SequenceHandler
extends SequenceHandler
{
   /**
    * The list of sequences grouped by databases. It uses lazy initialization.
    */
   private HashMap<String, LinkedList<String>> sequences = null;

   /**
    * The only constructor takes the name of the database as argument. 
    * It is passed on to super class.
    *
    * @param   ldbName
    *          The database logical name on which this handler will work with. 
    */
   public SQLServer2012SequenceHandler(String ldbName)
   {
      super(ldbName);
   }

   /**
    * Queries the current value of a sequence.
    * <p>
    * This method should be called only in a safe environment, where the database-level lock for
    * access to sequences has already been acquired.
    *
    * @param   seqName
    *          The name of the sequence.
    * @param   ldbName
    *          The database to which the sequence belongs to. It is assumed it is not null and
    *          valid.
    *
    * @return  the current value of the sequence
    */
   @Override
   protected int64 safeGetCurrentValue(String seqName, String ldbName)
   {
      // TODO
      return null;
   }

   /**
    * Computes and returns the next value of a sequence.
    * <p>
    * This method should be called only in a safe environment, where the database-level lock for
    * access to sequences has already been acquired.
    *
    * @param   seqName
    *          The name of the sequence.
    * @param   ldbName
    *          The database to which the sequence belongs to. It is assumed it is not null and
    *          valid.
    *
    * @return  the next value of the sequence or unknown value (?) if sequence/database not found
    *          or the sequence is not cycling and has passed the maximum/minimum value.
    */
   @Override
   protected int64 safeGetNextValue(String seqName, String ldbName)
   {
      // TODO
      return null;
   }

   /**
    * Initialize (reset) the current value of a sequence.
    * <p>
    * This method should be called only in a safe environment, where the database-level lock for
    * access to sequences has already been acquired.
    *
    * @param   seqName
    *          The name of the sequence.
    * @param   ldbName
    *          The database to which the sequence belongs to. 
    *          It is assumed it is not null and valid.
    * @param   newVal
    *          The value the sequence will be reseted to.
    */
   @Override
   protected void safeSetValue(String seqName, String ldbName, long newVal)
   {
      // TODO
   }

   /**
    * Test whether the database contains a sequence. This is useful to identify the correct
    * database/sequences when a 4GL static function / statement is called without the database
    * parameter.
    * <p>
    * This method should be called only in a safe environment, where the database-level lock for
    * access to sequences has already been acquired.
    *
    * @param   seqName
    *          The name of the sequence.
    * @param   ldbName
    *          The database to which the sequence belongs to. If is null then the first connected
    *          database is assumed.
    *
    * @return  true if the sequence is defined in the database
    */
   @Override
   protected boolean safeContains(String seqName, String ldbName)
   {
      // TODO
      return false;
   }
}
